# SPDX-FileCopyrightText: 2011-2023 Blender Foundation
#
# SPDX-License-Identifier: GPL-2.0-or-later
#
# Modified by PROTOWLF
#
# PROTO FBX Export Wrapper
# Wraps additional features around Blender's default FBX Exporter


# import, reload if already imported
if "proto_fbx_utils" not in locals():
    from . import proto_fbx_utils
    #print("PROTO Tools - proto_fbx_init: Importing")
else:
    import importlib
    proto_fbx_utils = importlib.reload(proto_fbx_utils)
    #print("PROTO Tools - proto_fbx_init: Reloading Scripts") 

from .proto_fbx_utils import (
    select_objects_for_action_export,
    get_export_objects,
    get_actions_for_armature,
    duplicate_export_objects,
    unit_scale_export_objects,
    apply_min_two_frames_to_actions,
    do_fbx_export,
    cleanup_scene
)


import bpy
from bpy.props import (
    StringProperty,
    BoolProperty,
    FloatProperty,
    EnumProperty,
    CollectionProperty,
)
from bpy_extras.io_utils import (
    ImportHelper,
    ExportHelper,
    orientation_helper,
    path_reference_mode,
    axis_conversion,
    poll_file_object_drop,
)
import time
import math


@orientation_helper(axis_forward='-Z', axis_up='Y')
class ProtoExportFBX(bpy.types.Operator, ExportHelper):
    """Open FBX file exporter dialogue"""
    bl_idname = "proto_export_scene.fbx"
    bl_label = "PROTO Export FBX"
    bl_options = {'UNDO', 'PRESET'}

    filename_ext = ".fbx"
    filter_glob: StringProperty(default="*.fbx", options={'HIDDEN'})

    # List of operator properties, the attributes will be assigned
    # to the class instance from the operator settings before calling.

    use_selection: BoolProperty(
        name="Selected Objects",
        description="Export selected and visible objects only",
        default=True,
    )
    use_visible: BoolProperty(
        name='Visible Objects',
        description='Export visible objects only',
        default=True
    )
    use_active_collection: BoolProperty(
        name="Active Collection",
        description="Export only objects from the active collection (and its children)",
        default=False,
    )
    collection: StringProperty(
        name="Source Collection",
        description="Export only objects from this collection (and its children)",
        default="",
    )
    global_scale: FloatProperty(
        name="Scale",
        description="Scale all data (Some importers do not support scaled armatures!)",
        min=0.001, max=1000.0,
        soft_min=0.01, soft_max=1000.0,
        default=1.0,
    )
    apply_unit_scale: BoolProperty(
        name="Apply Unit",
        description="Take into account current Blender units settings (if unset, raw Blender Units values are used as-is)",
        default=True,
    )
    apply_scale_options: EnumProperty(
        items=(('FBX_SCALE_NONE', "All Local",
                "Apply custom scaling and units scaling to each object transformation, FBX scale remains at 1.0"),
               ('FBX_SCALE_UNITS', "FBX Units Scale",
                "Apply custom scaling to each object transformation, and units scaling to FBX scale"),
               ('FBX_SCALE_CUSTOM', "FBX Custom Scale",
                "Apply custom scaling to FBX scale, and units scaling to each object transformation"),
               ('FBX_SCALE_ALL', "FBX All",
                "Apply custom scaling and units scaling to FBX scale"),
               ),
        name="Apply Scalings",
        description="How to apply custom and units scalings in generated FBX file "
        "(Blender uses FBX scale to detect units on import, "
        "but many other applications do not handle the same way)",
    )

    use_space_transform: BoolProperty(
        name="Use Space Transform",
        description="Apply global space transform to the object rotations. When disabled "
        "only the axis space is written to the file and all object transforms are left as-is",
        default=True,
    )
    bake_space_transform: BoolProperty(
        name="Apply Transform",
        description="Bake space transform into object data, avoids getting unwanted rotations to objects when "
        "target space is not aligned with Blender's space "
        "(WARNING! experimental option, use at own risk, known to be broken with armatures/animations)",
        default=False,
    )
    
    object_types: EnumProperty(
        name="Object Types",
        options={'ENUM_FLAG'},
        items=(('EMPTY', "Empty", ""),
               ('CAMERA', "Camera", ""),
               ('LIGHT', "Lamp", ""),
               ('ARMATURE', "Armature", "WARNING: not supported in dupli/group instances"),
               ('MESH', "Mesh", ""),
               ('OTHER', "Other", "Other geometry types, like curve, metaball, etc. (converted to meshes)"),
               ),
        description="Which kind of object to export",
        default={'EMPTY', 'CAMERA', 'LIGHT', 'ARMATURE', 'MESH', 'OTHER'},
    )

    use_mesh_modifiers: BoolProperty(
        name="Apply Modifiers",
        description="Apply modifiers to mesh objects (except Armature ones) - "
        "WARNING: prevents exporting shape keys",
        default=True,
    )
    use_mesh_modifiers_render: BoolProperty(
        name="Use Modifiers Render Setting",
        description="Use render settings when applying modifiers to mesh objects (DISABLED in Blender 2.8)",
        default=True,
    )
    mesh_smooth_type: EnumProperty(
        name="Smoothing",
        items=(('OFF', "Normals Only", "Export only normals instead of writing edge or face smoothing data"),
               ('FACE', "Face", "Write face smoothing"),
               ('EDGE', "Edge", "Write edge smoothing"),
               ),
        description="Export smoothing information "
        "(prefer 'Normals Only' option if your target importer understand split normals)",
        default='FACE',
    )
    colors_type: EnumProperty(
        name="Vertex Colors",
        items=(('NONE', "None", "Do not export color attributes"),
               ('SRGB', "sRGB", "Export colors in sRGB color space"),
               ('LINEAR', "Linear", "Export colors in linear color space"),
               ),
        description="Export vertex color attributes",
        default='SRGB',
    )
    prioritize_active_color: BoolProperty(
        name="Prioritize Active Color",
        description="Make sure active color will be exported first. Could be important "
        "since some other software can discard other color attributes besides the first one",
        default=False,
    )
    use_subsurf: BoolProperty(
        name="Export Subdivision Surface",
        description="Export the last Catmull-Rom subdivision modifier as FBX subdivision "
        "(does not apply the modifier even if 'Apply Modifiers' is enabled)",
        default=False,
    )
    use_mesh_edges: BoolProperty(
        name="Loose Edges",
        description="Export loose edges (as two-vertices polygons)",
        default=False,
    )
    use_tspace: BoolProperty(
        name="Tangent Space",
        description="Add binormal and tangent vectors, together with normal they form the tangent space "
        "(will only work correctly with tris/quads only meshes!)",
        default=False,
    )
    use_triangles: BoolProperty(
        name="Triangulate Faces",
        description="Convert all faces to triangles",
        default=False,
    )
    use_custom_props: BoolProperty(
        name="Custom Properties",
        description="Export custom properties",
        default=False,
    )
    add_leaf_bones: BoolProperty(
        name="Add Leaf Bones",
        description="Append a final bone to the end of each chain to specify last bone length "
        "(use this when you intend to edit the armature from exported data)",
        default=False  # False for commit!
    )
    primary_bone_axis: EnumProperty(
        name="Primary Bone Axis",
        items=(('X', "X Axis", ""),
               ('Y', "Y Axis", ""),
               ('Z', "Z Axis", ""),
               ('-X', "-X Axis", ""),
               ('-Y', "-Y Axis", ""),
               ('-Z', "-Z Axis", ""),
               ),
        default='Y',
    )
    secondary_bone_axis: EnumProperty(
        name="Secondary Bone Axis",
        items=(('X', "X Axis", ""),
               ('Y', "Y Axis", ""),
               ('Z', "Z Axis", ""),
               ('-X', "-X Axis", ""),
               ('-Y', "-Y Axis", ""),
               ('-Z', "-Z Axis", ""),
               ),
        default='X',
    )
    use_armature_deform_only: BoolProperty(
        name="Only Deform Bones",
        description="Only write deforming bones (and non-deforming ones when they have deforming children)",
        default=True,
    )
    armature_nodetype: EnumProperty(
        name="Armature FBXNode Type",
        items=(('NULL', "Null", "'Null' FBX node, similar to Blender's Empty (default)"),
               ('ROOT', "Root", "'Root' FBX node, supposed to be the root of chains of bones..."),
               ('LIMBNODE', "LimbNode", "'LimbNode' FBX node, a regular joint between two bones..."),
               ),
        description="FBX type of node (object) used to represent Blender's armatures "
        "(use the Null type unless you experience issues with the other app, "
        "as other choices may not import back perfectly into Blender...)",
        default='NULL',
    )
    bake_anim: BoolProperty(
        name="Baked Animation",
        description="Export baked keyframe animation",
        default=False,
    )
    bake_anim_use_all_bones: BoolProperty(
        name="Key All Bones",
        description="Force exporting at least one key of animation for all bones "
        "(needed with some target applications, like UE4)",
        default=True,
    )
    bake_anim_use_nla_strips: BoolProperty(
        name="NLA Strips",
        description="Export each non-muted NLA strip as a separated FBX's AnimStack, if any, "
        "instead of global scene animation. NOTE: if this is true, and no actions are pushed down into the NLA stack, then no animations will be exported in the FBX file!",
        default=False,
    )
    bake_anim_use_all_actions: BoolProperty(
        name="All Actions",
        description="Export each action as a separated FBX's AnimStack, instead of global scene animation "
        "(note that animated objects will get all actions compatible with them, "
        "others will get no animation at all)",
        default=True,
    )
    bake_anim_force_startend_keying: BoolProperty(
        name="Force Start/End Keying",
        description="Always add a keyframe at start and end of actions for animated channels",
        default=True,
    )
    bake_anim_step: FloatProperty(
        name="Sampling Rate",
        description="How often to evaluate animated values (in frames)",
        min=0.01, max=100.0,
        soft_min=0.1, soft_max=10.0,
        default=1.0,
    )
    bake_anim_simplify_factor: FloatProperty(
        name="Simplify",
        description="How much to simplify baked values (0.0 to disable, the higher the more simplified)",
        min=0.0, max=100.0,  # No simplification to up to 10% of current magnitude tolerance.
        soft_min=0.0, soft_max=10.0,
        default=1.0,  # default: min slope: 0.005, max frame step: 10.
    )
    path_mode: path_reference_mode
    embed_textures: BoolProperty(
        name="Embed Textures",
        description="Embed textures in FBX binary file (only for \"Copy\" path mode!)",
        default=False,
    )
    
    # Proto: BATCH MODE NOT SUPPORTED
    # batch_mode: EnumProperty(
    #     name="Batch Mode",
    #     items=(('OFF', "Off", "Active scene to file"),
    #            ('SCENE', "Scene", "Each scene as a file"),
    #            ('COLLECTION', "Collection",
    #             "Each collection (data-block ones) as a file, does not include content of children collections"),
    #            ('SCENE_COLLECTION', "Scene Collections",
    #             "Each collection (including master, non-data-block ones) of each scene as a file, "
    #             "including content from children collections"),
    #            ('ACTIVE_SCENE_COLLECTION', "Active Scene Collections",
    #             "Each collection (including master, non-data-block one) of the active scene as a file, "
    #             "including content from children collections"),
    #            ),
    # )
    #use_batch_own_dir: BoolProperty(
    #    name="Batch Own Dir",
    #    description="Create a dir for each exported file",
    #    default=True,
    #)
    use_metadata: BoolProperty(
        name="Use Metadata",
        default=True,
        options={'HIDDEN'},
    )
    
    # Proto Addition
    unit_scale_type: EnumProperty(
        name="Export Units",
        description="Bake in a conversion from Blender's default units (meters) to a different length unit. Use this when exporting for engines that don't handle unit conversions well on import",
        items=(('None', "None", "Do not bake a unit conversion"),
               ('Centimeters', "Centimeters (Unreal)", "Bake a unit conversion to centimeters. Use this when exporting for Unreal Engine"),
               ('Inches', "Inches (Source)", "Bake a unit conversion to inches. Use this when exporting for Source and Source 2 Engine"),
               ('Custom', "Custom", "Bake a custom conversion. Enter a multiplier to apply to meters (e.g. 100 for centimeters, 39.37 for inches)"),
               ),
        default='None',
    )
    
    # Proto Addition
    unit_scale_custom: FloatProperty(
        name="Custom Unit Conversion",
        description="A custom unit conversion. Enter a multiplier to apply to meters (e.g. 100 for centimeters, 39.37 for inches)",
        min=0.0001, max=10000.0,
        soft_min=0.0001, soft_max=10000.0,
        default=1.0,
    )
    
    # Proto Addition
    root_bone_name: StringProperty(
        name="Rename Root",
        description="Blender exports the armature itself as the root bone. You may rename it, or leave blank to use the existing name.\nNOTE: Unreal will strip out a root bone with name 'Armature'; if you set this to 'Armature', this will cause Unreal to use the first bone as the root when imported",
        default="",
    )
    
    # Proto Addition
    min_two_frames: BoolProperty(
        name="Minimum 2 Frames",
        description="When exporting Actions, force frame range to be at least 2 frames long.\nFixes import issue with Unreal, where it cannot import 1 frame animations",
        default=False,
    )
    
    # Proto Addition
    # Not exposed to UI -- used by Quick Export wrappers, to specify to use prototools.action_export_list
    one_file_per_action: BoolProperty(
		name="1 File Per Action",
        description="When exporting all Actions, save each action to its own FBX file. Each file will be named as the Action's name",
		default=False,
        options={'HIDDEN'},
	)
    

    def draw(self, context):
        layout = self.layout
        layout.use_property_split = True
        layout.use_property_decorate = False  # No animation.

        # Are we inside the File browser
        is_file_browser = context.space_data.type == 'FILE_BROWSER'

        export_main(layout, self, is_file_browser)
        export_panel_proto(layout, self)
        export_panel_include(layout, self, is_file_browser)
        export_panel_transform(layout, self)
        export_panel_geometry(layout, self)
        export_panel_armature(layout, self)
        export_panel_animation(layout, self)

    @property
    def check_extension(self):
        return True #self.batch_mode == 'OFF'

    def execute(self, context):
        from mathutils import Matrix
        
        print("\nPROTO FBX Exporter - beginnning export")
        
        # Verify filepath, but ignore it for one_file_per_action mode
        if self.one_file_per_action == False:
            if not self.filepath:
                raise Exception("filepath not set")
            
            # Validate filepath
            if self.filepath == "" or self.filepath == "/":
                self.report({'ERROR'}, "Invalid File Path")
                return {'CANCELLED'}
                
            # We need a file name
            if self.filepath[-1] == '/' or self.filepath[-1] == '\\':
                self.report({'ERROR'}, "Invalid File Path - Must specify a file name")
                return {'CANCELLED'}
    
            # Add .fbx suffix if missing
            split_filepath = self.filepath.split(".")
            if split_filepath[-1] != "fbx":
                self.filepath += ".fbx"
        
        global_matrix = (axis_conversion(to_forward=self.axis_forward,
                                         to_up=self.axis_up,
                                         ).to_4x4()
                         if self.use_space_transform else Matrix())

        keywords = self.as_keywords(ignore=("check_existing",
                                            "filter_glob",
                                            "ui_tab",
                                            ))

        keywords["global_matrix"] = global_matrix
        
        # Determine unit scale for export
        self.final_unit_scale = 1.0
        if keywords["unit_scale_type"] == "Centimeters":
            self.final_unit_scale = 100.0
        if keywords["unit_scale_type"] == "Inches":
            self.final_unit_scale = 39.37
        if keywords["unit_scale_type"] == "Custom":
            self.final_unit_scale = keywords["unit_scale_custom"]
        
        # Cache selection at start of export
        self.original_selected_objects = bpy.context.selected_objects
        self.original_active_selection = bpy.context.view_layer.objects.active
        
        self.saved_unit_settings = {}
        self.export_objects = []
        self.duplicate_objects = []
        self.armature_original_name = ""
        self.scale_changed_actions = []
        self.original_action_frame_ranges = {}
        try:
            #start_time = time.time()
            
            # switch to Object Mode
            try:
                bpy.ops.object.mode_set(mode='OBJECT')
            except:
                pass
    
            
            # Get objects for export
            self.export_objects = get_export_objects(context, keywords["use_selection"], keywords["use_visible"], keywords["use_active_collection"], keywords["collection"])
            
            # Verify objects are valid
            if not self.export_objects:
                self.report({'ERROR'}, "no valid exportable objects found")
                return {'CANCELLED'}
            else:
                num_armatures = 0
                for ob in self.export_objects:
                    if ob.type == 'ARMATURE':
                        num_armatures += 1
                if num_armatures > 1:
                    self.report({'ERROR'}, "Multiple Armatures in export! Only 1 armatures is supported.")
                    return {'CANCELLED'}
            
            # store current scene unit settings
            for prop_name in dir(context.scene.unit_settings):
                if not (prop_name in ['bl_rna', 'rna_type'] or prop_name.startswith('__')):
                    self.saved_unit_settings[prop_name] = getattr(context.scene.unit_settings, prop_name)
            
            # Duplicate objects
            self.duplicate_objects, self.armature_original_name = duplicate_export_objects(self.export_objects, keywords["root_bone_name"])
            #print("Duplicated objects:")
            #for ob in self.duplicate_objects:
            #    print(ob.name)
            
            # Apply unit scaling correction
            if self.final_unit_scale != 1.0:
                unit_scale_export_objects(self.final_unit_scale, self.duplicate_objects, self.scale_changed_actions)
            
            # Select Duplicates, keep track of armatures
            armatures = []
            bpy.ops.object.select_all(action='DESELECT')
            for ob in self.duplicate_objects:
                bpy.context.view_layer.objects.active = ob
                ob.select_set(state=True)
                if ob.type == "ARMATURE":
                    armatures.append(ob)
            
            #end_time = time.time()
            #print("PROTO FBX Exporter - DEBUG TIMER - Scene Preparation took: ", end_time-start_time)
            #start_time = time.time()
            
            if keywords["one_file_per_action"]:
                # One file per action mode -- runs multiple exports
                prototools = context.scene.prototools
                
                # Verify parameters
                if keywords["bake_anim"] != True:
                    self.report({'ERROR'}, "Running in one_file_per_action mode, but bake_anim was False! This is not allowed.")
                    return {'CANCELLED'}
                if keywords["bake_anim_use_all_actions"] != False:
                    self.report({'ERROR'}, "Running in one_file_per_action mode, but bake_anim_use_all_actions was True! This is not allowed.")
                    return {'CANCELLED'}
                if len(prototools.action_export_list) == 0:
                    self.report({'ERROR'}, "Running in one_file_per_action mode, but prototools.action_export_list was empty!")
                    return {'CANCELLED'}
                
                # We must have exactly 1 armature as part of this export
                if len(armatures) == 0:
                    self.report({'ERROR'}, "Running in one_file_per_action mode, but no armatures were present in exported objects!")
                    return {'CANCELLED'}
                if len(armatures) > 1:
                    self.report({'ERROR'}, "Running in one_file_per_action mode, but more than 1 armature was present in exported objects! Only 1 is supported.")
                    return {'CANCELLED'}
                armature = armatures[0]
                
                print("PROTO FBX Exporter - Beginning one-file-per-action exports")
                for action_export in prototools.action_export_list:
                    # Set active action
                    armature.animation_data.action = action_export.action
                    
                    # Set timeline to the action's frame range
                    frame_start, frame_end = action_export.action.frame_range
                    bpy.context.scene.frame_start = math.floor(frame_start)
                    bpy.context.scene.frame_end = math.floor(frame_end)
                    
                    # if using min_two_frames, adjust timeline if necessary
                    if bpy.context.scene.frame_start == bpy.context.scene.frame_end and self.min_two_frames == True:
                        bpy.context.scene.frame_end += 1
                    
                    # Select objects for this action (may select meshes with shapekeys)
                    select_objects_for_action_export(context, armature, action_export.action)
                    
                    # Trigger FBX export for this action
                    self.filepath = action_export.export_filepath
                    do_fbx_export(self.filepath, keywords)
                
            else:
                # if using min_two_frames, we have to iterate actions and adjust their manual frame range
                # Assume 1 armature exported at a time
                if self.min_two_frames == True and len(armatures) > 0:
                    if len(armatures) > 1:
                        self.report({'ERROR'}, "Running with min_two_frames enabled, but more than 1 armature was present in exported objects! Only 1 is supported.")
                        return {'CANCELLED'}
                    apply_min_two_frames_to_actions(armatures[0], self.original_action_frame_ranges)
                
                # Trigger FBX export of selected objects
                do_fbx_export(self.filepath, keywords)
            
            #end_time = time.time()
            #print("PROTO FBX Exporter - DEBUG TIMER - internal export took: ", end_time-start_time)
            
        finally:
            # Clean up duplicates and modifications to scene
            cleanup_scene(context, self.final_unit_scale, self.saved_unit_settings, self.export_objects, self.duplicate_objects, self.armature_original_name, self.scale_changed_actions, self.original_action_frame_ranges)
            
            # Re-select objects from the start of the export
            bpy.ops.object.select_all(action='DESELECT')
            for ob in self.original_selected_objects:
                ob.select_set(state=True)
            bpy.context.view_layer.objects.active = self.original_active_selection
        
        # Warn users if any meshes had shapekeys, and we exported with modifiers applied
        if self.use_mesh_modifiers:
            found_shapekeys_with_mods = False
            for ob in self.export_objects:
                if ob.type == 'MESH' and ob.data.shape_keys != None:
                    found_non_arm_mod = False
                    for modifier in ob.modifiers:
                        if modifier.type != "ARMATURE":
                            found_non_arm_mod = True
                    if found_non_arm_mod:
                        found_shapekeys_with_mods = True
                        break
            if found_shapekeys_with_mods:
                msg = "Shapekeys may not have exported!\nApplying Modifiers prevents shapekey export."
                bpy.ops.prototools.infopopup('INVOKE_DEFAULT', text=msg)
                print("PROTO FBX Exporter - Warning - " + msg)
        
        print("PROTO FBX Exporter - export successful")
        return {'FINISHED'}


def export_main(layout, operator, is_file_browser):
    row = layout.row(align=True)
    row.prop(operator, "path_mode")
    sub = row.row(align=True)
    sub.enabled = (operator.path_mode == 'COPY')
    sub.prop(operator, "embed_textures", text="", icon='PACKAGE' if operator.embed_textures else 'UGLYPACKAGE')
    # PROTO: BATCH MODE NOT SUPPORTED
    #if is_file_browser:
    #    row = layout.row(align=True)
    #    row.prop(operator, "batch_mode")
    #    sub = row.row(align=True)
    #    sub.prop(operator, "use_batch_own_dir", text="", icon='NEWFOLDER')


def export_panel_proto(layout, operator):
    header, body = layout.panel("FBX_export_proto", default_closed=False)
    header.label(text="PROTO Options")
    if body:
        body.prop(operator, "unit_scale_type")
        if operator.unit_scale_type == "Custom":
            body.prop(operator, "unit_scale_custom")
        body.prop(operator, "root_bone_name")


def export_panel_include(layout, operator, is_file_browser):
    header, body = layout.panel("FBX_export_include", default_closed=False)
    header.label(text="Include")
    if body:
        sublayout = body.column(heading="Limit to")
        #sublayout.enabled = (operator.batch_mode == 'OFF')
        if is_file_browser:
            sublayout.prop(operator, "use_selection")
            sublayout.prop(operator, "use_visible")
            sublayout.prop(operator, "use_active_collection")

        body.column().prop(operator, "object_types")
        body.prop(operator, "use_custom_props")


def export_panel_transform(layout, operator):
    header, body = layout.panel("FBX_export_transform", default_closed=True)
    header.label(text="Transform")
    if body:
        body.prop(operator, "global_scale")
        body.prop(operator, "apply_scale_options")

        body.prop(operator, "axis_forward")
        body.prop(operator, "axis_up")

        body.prop(operator, "apply_unit_scale")
        body.prop(operator, "use_space_transform")
        row = body.row()
        row.prop(operator, "bake_space_transform")
        row.label(text="", icon='ERROR')
        
        
def export_panel_geometry(layout, operator):
    header, body = layout.panel("FBX_export_geometry", default_closed=True)
    header.label(text="Geometry")
    if body:
        body.prop(operator, "mesh_smooth_type")
        body.prop(operator, "use_subsurf")
        body.prop(operator, "use_mesh_modifiers")
        #sub = body.row()
        # sub.enabled = operator.use_mesh_modifiers and False  # disabled in 2.8...
        #sub.prop(operator, "use_mesh_modifiers_render")
        body.prop(operator, "use_mesh_edges")
        body.prop(operator, "use_triangles")
        sub = body.row()
        # ~ sub.enabled = operator.mesh_smooth_type in {'OFF'}
        sub.prop(operator, "use_tspace")
        body.prop(operator, "colors_type")
        body.prop(operator, "prioritize_active_color")


def export_panel_armature(layout, operator):
    header, body = layout.panel("FBX_export_armature", default_closed=True)
    header.label(text="Armature")
    if body:
        body.prop(operator, "primary_bone_axis")
        body.prop(operator, "secondary_bone_axis")
        body.prop(operator, "armature_nodetype")
        body.prop(operator, "use_armature_deform_only")
        body.prop(operator, "add_leaf_bones")


def export_panel_animation(layout, operator):
    header, body = layout.panel("FBX_export_bake_animation", default_closed=True)
    header.use_property_split = False
    header.prop(operator, "bake_anim", text="")
    header.label(text="Animation")
    if body:
        body.enabled = operator.bake_anim
        body.prop(operator, "bake_anim_use_all_bones")
        body.prop(operator, "bake_anim_use_nla_strips")
        body.prop(operator, "bake_anim_use_all_actions")
        body.prop(operator, "bake_anim_force_startend_keying")
        body.prop(operator, "bake_anim_step")
        body.prop(operator, "bake_anim_simplify_factor")


def menu_func_export(self, context):
    self.layout.operator(ProtoExportFBX.bl_idname, text="PROTO FBX (.fbx)")


def register():
    bpy.utils.register_class(ProtoExportFBX)
    bpy.types.TOPBAR_MT_file_export.append(menu_func_export)


def unregister():
    bpy.types.TOPBAR_MT_file_export.remove(menu_func_export)
    bpy.utils.unregister_class(ProtoExportFBX)


if __name__ == "__main__":
    register()
