# PROTO Tools Operators
# 2024 PROTOWLF, Licensed under GPL-3.0


# import, reload if already imported
if "proto_fbx_utils" not in locals():
    from .fbx_export import proto_fbx_utils
else:
    import importlib
    proto_fbx_utils = importlib.reload(proto_fbx_utils)

from .fbx_export.proto_fbx_utils import (
    get_actions_for_armature,
    action_exports_shapekeys,
    select_objects_for_action_export
)


import bpy
from bpy.props import (
    StringProperty
)
import os
import math
import time

def get_action_filepath(folderpath, action_name, context):
    prototools = context.scene.prototools
    
    if prototools.action_name_style == "Action":
        return folderpath + action_name + ".fbx"
    if prototools.action_name_style == "Name_Action":
        return folderpath + prototools.action_name_sharedname + "_" + action_name + ".fbx"
    if prototools.action_name_style == "Name-Action":
        return folderpath + prototools.action_name_sharedname + "-" + action_name + ".fbx"
    if prototools.action_name_style == "Name@Action":
        return folderpath + prototools.action_name_sharedname + "@" + action_name + ".fbx"
    




class ProtoTools_QuickExportMesh(bpy.types.Operator):
    """Export selected objects as an FBX mesh. May be static or skeletal (containing armature)"""
    bl_idname = "prototools.quick_export_mesh"
    bl_label = "Quick Export Mesh"
    
    def execute(self, context):
        prototools = context.scene.prototools
        
        # Validate filepath (with file name)
        if prototools.mesh_path == "" or prototools.mesh_path == "/":
            self.report({'ERROR'}, "Invalid File Path - Set a path above.")
            return {'CANCELLED'}
        if prototools.mesh_path[-1] == '/' or prototools.mesh_path[-1] == '\\':
            self.report({'ERROR'}, "Invalid File Path - Must specify a file name.")
            return {'CANCELLED'}
            
        # Must have objects selected
        if context.selected_objects == None or len(context.selected_objects) == 0:
            self.report({'ERROR'}, "Invalid Selection - No Objects selected.")
            return {'CANCELLED'}
            
        # Must have at least 1 mesh selected, maximum 1 armature
        found_mesh = False
        num_armatures = 0
        for ob in context.selected_objects:
            if ob.type == "MESH":
                found_mesh = True
            elif ob.type == "ARMATURE":
                num_armatures += 1
        if found_mesh == False:
            self.report({'ERROR'}, "Invalid Selection - No Meshes selected.")
            return {'CANCELLED'}
        if num_armatures > 1:
            self.report({'ERROR'}, "Invalid Selection - Multiple Armatures selected, maximum 1 Armature allowed.")
            return {'CANCELLED'}
        
        self.original_mode = context.mode
        
        try:
            # Switch to Object Mode
            try:
                bpy.ops.object.mode_set(mode='OBJECT')
            except:
                pass
            
            bpy.ops.proto_export_scene.fbx(
                filepath = prototools.mesh_path,
                # Include
                use_selection = True,
                use_visible = True,
                use_active_collection = False,
                collection = "",
                object_types = {'EMPTY', 'CAMERA', 'LIGHT', 'ARMATURE', 'MESH', 'OTHER'},
                use_custom_props = False, # Don't need to export curve data (e.g. shapekey animation)
                # Geometry
                use_mesh_modifiers  = prototools.apply_modifiers,
                # Bake Animations
                bake_anim = False,
                bake_anim_use_all_actions = False,
                # Proto Options
                unit_scale_type = prototools.unit_scale_type,
                unit_scale_custom = prototools.unit_scale_custom,
                root_bone_name = prototools.root_bone_name,
            )
        finally:
            # Restore previous mode
            try:
                bpy.ops.object.mode_set(mode=self.original_mode)
            except:
                pass
        
        return {'FINISHED'}


class ProtoTools_QuickExportCurAction(bpy.types.Operator):
    """Export current Action on selected Armature as an FBX animation.\nDo not select the bound Mesh, unless the Mesh has Shape Key animation\nNOTE: will export the Action's frame range, not the current Timeline"""
    bl_idname = "prototools.quick_export_curaction"
    bl_label = "Quick Export Current Action"
    
    def execute(self, context):
        prototools = context.scene.prototools
        
        # Validate filepath
        if prototools.anim_path == "" or prototools.anim_path == "/":
            self.report({'ERROR'}, "Invalid File Path - Set a path above.")
            return {'CANCELLED'}
        
        # Should be a path to a folder
        if prototools.anim_path[-1] != '/' and prototools.anim_path[-1] != '\\':
            self.report({'ERROR'}, "Invalid File Path - Must specify a folder name.")
            return {'CANCELLED'}
        
        # Must have exactly 1 armature selected
        # NOTE: we could exclude meshes here, but for shapekeys in animations we need to export the mesh as well
        armature = None
        for ob in context.selected_objects:
            if ob.type == 'ARMATURE':
                if armature != None:
                    self.report({'ERROR'}, "Multiple Armatures selected - Must select exactly 1 Armature.")
                    return {'CANCELLED'}
                armature = ob
        if armature == None:
            self.report({'ERROR'}, "No Armature selected - Must select exactly 1 Armature.")
            return {'CANCELLED'}
        
        # Must have an active action
        if armature.animation_data == None or armature.animation_data.action == None:
            self.report({'ERROR'}, "No active Action on selected Armature.")
            return {'CANCELLED'}
        action = armature.animation_data.action
        
        # Cache selection at start of export
        self.original_selected_objects = bpy.context.selected_objects
        self.original_active_selection = bpy.context.view_layer.objects.active
        
        action_name = armature.animation_data.action.name
        final_file_path = get_action_filepath(prototools.anim_path, action_name, context)
        
        self.original_frame_start = bpy.context.scene.frame_start
        self.original_frame_end = bpy.context.scene.frame_end
        self.original_mode = context.mode
        
        try:
            # Switch to Object Mode
            try:
                bpy.ops.object.mode_set(mode='OBJECT')
            except:
                pass
            
            # Select objects for this action (may select meshes with shapekeys)
            select_objects_for_action_export(context, armature, action)
            
            # Set timeline to the action's frame range
            frame_start, frame_end = armature.animation_data.action.frame_range
            bpy.context.scene.frame_start = math.floor(frame_start)
            bpy.context.scene.frame_end = math.floor(frame_end)
            
            # if using min_two_frames, adjust timeline if necessary
            if bpy.context.scene.frame_start == bpy.context.scene.frame_end and prototools.min_two_frames == True:
                bpy.context.scene.frame_end += 1
            
            # Do the export
            bpy.ops.proto_export_scene.fbx(
                filepath = final_file_path,
                # Include
                use_selection = True,
                use_visible = True,
                use_active_collection = False,
                collection = "",
                object_types = {'EMPTY', 'CAMERA', 'LIGHT', 'ARMATURE', 'MESH', 'OTHER'},
                use_custom_props = True, # Export curve data (e.g. shapekey animation)
                # Geometry
                use_mesh_modifiers = False, # ignore prototools.apply_modifiers for anim exports
                # Bake Animations
                bake_anim = True,
                bake_anim_use_all_actions = False,
                # Proto Options
                unit_scale_type = prototools.unit_scale_type,
                unit_scale_custom = prototools.unit_scale_custom,
                root_bone_name = prototools.root_bone_name,
                min_two_frames = False # Already handled it, so don't let PROTO exporter do this!
            )
        
        finally:
            # Reset the original timeline frame range
            bpy.context.scene.frame_start = self.original_frame_start
            bpy.context.scene.frame_end = self.original_frame_end
            
            # Re-select objects from the start of the export
            bpy.ops.object.select_all(action='DESELECT')
            for ob in self.original_selected_objects:
                ob.select_set(state=True)
            bpy.context.view_layer.objects.active = self.original_active_selection
            
            # Restore previous mode
            try:
                bpy.ops.object.mode_set(mode=self.original_mode)
            except:
                pass
        
        return {'FINISHED'}
        

def cache_pose(armature):
    saved_pose = {}
    for pose_bone in armature.pose.bones:
        saved_pose[pose_bone.name] = pose_bone.location.copy(), pose_bone.scale.copy(), pose_bone.rotation_mode, pose_bone.rotation_quaternion.copy(), pose_bone.rotation_euler.copy()
    return saved_pose
    
def restore_pose(armature, saved_pose):
    for pose_bone in armature.pose.bones:
        location, scale, rotation_mode, rotation_quaternion, rotation_euler = saved_pose[pose_bone.name]
        pose_bone.location = location
        pose_bone.scale = scale
        pose_bone.rotation_mode = rotation_mode
        pose_bone.rotation_quaternion = rotation_quaternion
        pose_bone.rotation_euler = rotation_euler
    


class ProtoTools_QuickExportAllActions(bpy.types.Operator):
    """Export all Actions on selected Armature as FBX animations.\nDo not select the bound Mesh, unless the Mesh has Shape Key animation"""
    bl_idname = "prototools.quick_export_allactions"
    bl_label = "Quick Export All Actions"
    
    def execute(self, context):
        prototools = context.scene.prototools
        
        # We use a different path property with one_file_per_action
        filepath = prototools.all_anims_path
        filepath_folder = prototools.all_anims_path_folder
        
        # Validate file / folder
        if prototools.one_file_per_action:
            if filepath_folder == "" or filepath_folder == "/":
                self.report({'ERROR'}, "Invalid File Path - Set a path above.")
                return {'CANCELLED'}
            
            # Should be a path to a folder
            if filepath_folder[-1] != '/' and filepath_folder[-1] != '\\':
                self.report({'ERROR'}, "Invalid File Path - With 1 File Per Action, must specify a folder name.")
                return {'CANCELLED'}
        else:
            if filepath == "" or filepath == "/":
                self.report({'ERROR'}, "Invalid File Path - Set a path above.")
                return {'CANCELLED'}
            
            # Should be a path to a file
            if filepath[-1] == '/' or filepath[-1] == '\\':
                self.report({'ERROR'}, "Invalid File Path - Must specify a file name.")
                return {'CANCELLED'}
                
        # Must have exactly 1 armature selected
        armature = None
        for ob in context.selected_objects:
            if ob.type == 'ARMATURE':
                if armature != None:
                    self.report({'ERROR'}, "Multiple Armatures selected - Must select exactly 1 Armature.")
                    return {'CANCELLED'}
                armature = ob
        if armature == None:
            self.report({'ERROR'}, "No Armature selected - Must select exactly 1 Armature.")
            return {'CANCELLED'}
        
        # Make sure armature has bones
        if armature.pose == None or len(armature.pose.bones) == 0:
            self.report({'ERROR'}, "Invalid Armature - Must have at least 1 bone.")
            return {'CANCELLED'}
        
        actions = get_actions_for_armature(armature)
        
        if len(actions) == 0:
            self.report({'ERROR'}, "Found no Actions with keyframes manipulating the selected Armature.")
            return {'CANCELLED'}
        
        # Cache information to reset later
        self.original_active_action = armature.animation_data.action
        self.original_object_mode = bpy.context.object.mode
        self.original_pose = cache_pose(armature)
        self.original_frame_start = bpy.context.scene.frame_start
        self.original_frame_end = bpy.context.scene.frame_end
        self.original_selected_objects = bpy.context.selected_objects
        self.original_active_selection = bpy.context.view_layer.objects.active
        self.original_mode = context.mode
        
        # Run the export(s)
        try:
            # Switch to Object Mode
            try:
                bpy.ops.object.mode_set(mode='OBJECT')
            except:
                pass
            
            # Select objects for export
            # if any action we're exporting has shapekey animation, we need to select meshes with shapekeys
            # NOTE: since we only support 1 armature, all actions with shapekeys requires the same object(s) selected
            for action in actions:
                if action_exports_shapekeys(action):
                    select_objects_for_action_export(context, armature, action)
                    break;
            
            if prototools.one_file_per_action:
                # Create a list of action exports to send to the PROTO fbx exporter so it can export them individually
                prototools.action_export_list.clear()
                for action in actions:
                    action_export = prototools.action_export_list.add()
                    action_export.action = action
                    
                    # Get final fbx name
                    action_export.export_filepath = get_action_filepath(filepath_folder, action.name, context)
                    
                # Run one export in one_file_per_action mode
                bpy.ops.proto_export_scene.fbx(
                    filepath = "", # NOTE: will be ignored in one_file_per_action mode
                    # Include
                    use_selection = True,
                    use_visible = True,
                    use_active_collection = False,
                    collection = "",
                    object_types = {'EMPTY', 'CAMERA', 'LIGHT', 'ARMATURE', 'MESH', 'OTHER'},
                    use_custom_props = True, # Export curve data (e.g. shapekey animation)
                    # Geometry
                    use_mesh_modifiers = False, # ignore prototools.apply_modifiers for anim exports
                    # Bake Animations
                    bake_anim = True,
                    bake_anim_use_all_actions = False, # Must set False for one_file_per_action mode
                    # Proto Options
                    unit_scale_type = prototools.unit_scale_type,
                    unit_scale_custom = prototools.unit_scale_custom,
                    root_bone_name = prototools.root_bone_name,
                    one_file_per_action = True, # Tells proto_export_scene.fbx to use prototools.action_export_list
                    min_two_frames = prototools.min_two_frames # PROTO exporter handles min_two_frames
                )
                
            else:
                # Run one export with bake_anim_use_all_actions
                bpy.ops.proto_export_scene.fbx(
                    filepath = filepath,
                    # Include
                    use_selection = True,
                    use_visible = True,
                    use_active_collection = False,
                    collection = "",
                    object_types = {'EMPTY', 'CAMERA', 'LIGHT', 'ARMATURE', 'MESH', 'OTHER'},
                    use_custom_props = True, # Export curve data (e.g. shapekey animation)
                    # Geometry
                    use_mesh_modifiers = False, # ignore prototools.apply_modifiers for anim exports
                    # Bake Animations
                    bake_anim = True,
                    bake_anim_use_all_actions = True, # TRUE, DOING ALL ACTIONS IN ONE
                    # Proto Options
                    unit_scale_type = prototools.unit_scale_type,
                    unit_scale_custom = prototools.unit_scale_custom,
                    root_bone_name = prototools.root_bone_name,
                    min_two_frames = prototools.min_two_frames # PROTO exporter handles min_two_frames
                )
        
        finally:
            # Reset the original active action
            armature.animation_data.action = self.original_active_action
            
            # Reset the original pose
            #bpy.ops.object.mode_set(mode='POSE')
            #armature.pose = self.original_pose
            #bpy.ops.object.mode_set(self.original_object_mode)
            restore_pose(armature, self.original_pose)
            
            # Reset the original timeline frame range
            bpy.context.scene.frame_start = self.original_frame_start
            bpy.context.scene.frame_end = self.original_frame_end
            
            # Re-select objects from the start of the export
            bpy.ops.object.select_all(action='DESELECT')
            for ob in self.original_selected_objects:
                ob.select_set(state=True)
            bpy.context.view_layer.objects.active = self.original_active_selection
            
            # Restore previous mode
            try:
                bpy.ops.object.mode_set(mode=self.original_mode)
            except:
                pass
            
            
        return {'FINISHED'}
        


class ProtoTools_MakeAnimData(bpy.types.Operator):
    """Create Anim Data for an Object"""
    bl_idname = "prototools.makeanimdata"
    bl_label = "Create Anim Data"
    
    @classmethod
    def poll(cls, context):
        if context.active_object != None and context.active_object.animation_data == None:
            return True
    
    def execute(self, context):
        ob = context.active_object
        if ob.animation_data == None:
            ob.animation_data_create()
        
        return {'FINISHED'}


class ProtoTools_NewAction(bpy.types.Operator):
    """Create New Action"""
    bl_idname = "prototools.actionnew"
    bl_label = "New Action"
    
    @classmethod
    def poll(cls, context):
        if context.active_object != None:
            return True
    
    def execute(self, context):
        ob = context.active_object
        new_action = None
        
        if ob.animation_data != None and ob.animation_data.action != None:
            # if the object already has an action assigned, duplicate it
            new_action = ob.animation_data.action.copy()
        else:
            # if not, create a new action
            new_action_name = "Action"
            new_action = bpy.data.actions.new(new_action_name)
            if ob.animation_data == None:
                ob.animation_data_create()
        
        # Set fake user on the new action (convenience to avoid accidentally losing actions)
        new_action.use_fake_user = True
        
        ob.animation_data.action = new_action
        
        return {'FINISHED'}


class ProtoTools_UnlinkAction(bpy.types.Operator):
    """Unlink this action from the active action slot"""
    bl_idname = "prototools.actionunlink"
    bl_label = "Unlink Action"
    
    @classmethod
    def poll(cls, context):
        if context.active_object != None:
            return True
    
    def execute(self, context):
        ob = context.active_object
        if ob.animation_data != None and ob.animation_data.action != None:
            ob.animation_data.action = None
        
        return {'FINISHED'}
        

class ProtoTools_DeleteAction(bpy.types.Operator):
    """Delete this action from the Blender file - WARNING: this will permanently delete the action!"""
    bl_idname = "prototools.actiondelete"
    bl_label = "Delete Action"
    
    @classmethod
    def poll(cls, context):
        if context.active_object != None:
            return True
    
    def execute(self, context):
        ob = context.active_object
        if ob.animation_data != None and ob.animation_data.action != None:
            action = ob.animation_data.action
            bpy.data.actions.remove(action, do_unlink = True)
        
        return {'FINISHED'}
        
    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self)
        

class ProtoTools_TimelineToAction(bpy.types.Operator):
    """Set the Scene Timeline to match the start and end of the current Action"""
    bl_idname = "prototools.timelinetoaction"
    bl_label = "Set Timeline"
    
    @classmethod
    def poll(cls, context):
        if context.active_object != None:
            return True
    
    def execute(self, context):
        prototools = context.scene.prototools
        ob = context.active_object
        if ob.animation_data != None and ob.animation_data.action != None:
            action = ob.animation_data.action
            frame_start, frame_end = action.frame_range
            if prototools.set_timeline_ignore_zero == True and frame_start < 1 and frame_end >= 2:
                frame_start = 1
            bpy.context.scene.frame_start = math.floor(frame_start)
            bpy.context.scene.frame_end = math.floor(frame_end)
        
        return {'FINISHED'}
        

class ProtoTools_InfoPopup(bpy.types.Operator):
    """Pop up a dialog with information. No functionality, the only interaction the user has is closing it"""
    bl_idname = "prototools.infopopup"
    bl_label = "Info"
    
    text : StringProperty(default="")
    icon : StringProperty(default='INFO')
    
    def draw(self, context):
        layout = self.layout
        
        # Display multi-line text
        text_lines = self.text.split('\n')
        did_icon = False
        for line in text_lines:
            if did_icon:
                layout.label(text=line)
            else:
                layout.label(text=line, icon=self.icon)
                did_icon = True
        
        props = layout.template_popup_confirm("", text="", cancel_text="OK")

    def execute(self, context):
        return {"FINISHED"}

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self, width=400)
        

def register():
    bpy.utils.register_class(ProtoTools_QuickExportMesh)
    bpy.utils.register_class(ProtoTools_QuickExportCurAction)
    bpy.utils.register_class(ProtoTools_QuickExportAllActions)
    
    bpy.utils.register_class(ProtoTools_MakeAnimData)
    bpy.utils.register_class(ProtoTools_NewAction)
    bpy.utils.register_class(ProtoTools_UnlinkAction)
    bpy.utils.register_class(ProtoTools_DeleteAction)
    bpy.utils.register_class(ProtoTools_TimelineToAction)
    bpy.utils.register_class(ProtoTools_InfoPopup)
    

def unregister():
    bpy.utils.unregister_class(ProtoTools_QuickExportMesh)
    bpy.utils.unregister_class(ProtoTools_QuickExportCurAction)
    bpy.utils.unregister_class(ProtoTools_QuickExportAllActions)
    
    bpy.utils.unregister_class(ProtoTools_MakeAnimData)
    bpy.utils.unregister_class(ProtoTools_NewAction)
    bpy.utils.unregister_class(ProtoTools_UnlinkAction)
    bpy.utils.unregister_class(ProtoTools_DeleteAction)
    bpy.utils.unregister_class(ProtoTools_TimelineToAction)
    bpy.utils.unregister_class(ProtoTools_InfoPopup)
    