# PROTO Tools UI Panels
# 2024 PROTOWLF, Licensed under GPL-3.0

import bpy
from bpy.props import (
    StringProperty,
    BoolProperty,
    FloatProperty,
    EnumProperty,
    CollectionProperty,
    PointerProperty,
)


def cur_action_shapekey_data_update(self, context):
    if self.cur_action_shapekey_data == True:
        if context.active_object != None and context.active_object.animation_data != None and context.active_object.animation_data.action != None:
            action = context.active_object.animation_data.action
            action["Export Shapekey Animation"] = self.cur_action_shapekey_data
            action.id_properties_ensure()  # Make sure the manager is updated
            ui = action.id_properties_ui("Export Shapekey Animation")
            ui.update(description="Specify whether this Action should export Shapekey animation with Quick Export functions.\nWhen True, Quick Export will include deformed meshes with Shapekeys in the exported FBX animation")
        self.cur_action_shapekey_data = False


class ActionExport(bpy.types.PropertyGroup):
    action: PointerProperty(type=bpy.types.Action)
    export_filepath: StringProperty()


class ProtoToolsProperties(bpy.types.PropertyGroup):

    def copy_from(self, old):
        self.unit_scale_type = old.unit_scale_type
        self.unit_scale_custom = old.unit_scale_custom
        self.root_bone_name = old.root_bone_name
        self.apply_modifiers = old.apply_modifiers
        self.mesh_path = old.mesh_path
        self.anim_path = old.anim_path
        self.all_anims_path = old.all_anims_path
        self.all_anims_path_folder = old.all_anims_path_folder
        self.one_file_per_action = old.one_file_per_action
        self.action_name_style = old.action_name_style
        self.action_name_sharedname = old.action_name_sharedname

    unit_scale_type: EnumProperty(
        name="Export Units",
        description="Bake in a conversion from Blender's default units (meters) to a different length unit. Use this when exporting for apps that don't handle unit conversions well on-import",
        items=(('None', "None", "Do not bake a unit conversion"),
               ('Centimeters', "Centimeters (Unreal)", "Bake a unit conversion to centimeters. Use this when exporting for Unreal Engine"),
               ('Inches', "Inches (Source)", "Bake a unit conversion to inches. Use this when exporting for Source and Source 2 Engine"),
               ('Custom', "Custom", "Bake a custom conversion. Enter a multiplier to apply to meters (e.g. 100 for centimeters, 39.37 for inches)"),
               ),
        default='None',
    )
    
    unit_scale_custom: FloatProperty(
        name="Custom Unit Conversion",
        description="A custom unit conversion. Enter a multiplier to apply to meters (e.g. 100 for centimeters, 39.37 for inches)",
        min=0.0001, max=10000.0,
        soft_min=0.0001, soft_max=10000.0,
        default=1.0,
    )
    
    root_bone_name: StringProperty(
        name="Rename Root",
        description="Blender exports the armature itself as the root bone. You may rename it, or leave blank to use the existing name.\nNOTE: Unreal will strip out a root bone with name 'Armature'; if you set this to 'Armature', this will cause Unreal to use the first bone as the root when imported",
        default="",
    )
    
    apply_modifiers: BoolProperty(
        name="Apply Modifiers",
        description="Apply modifiers to mesh objects (except Armature ones) - WARNING: prevents exporting shape keys\nNOTE: Only affects Quick Export mesh exports -- animations will never apply modifiers.",
        default=False
    )
    
    mesh_path: StringProperty(
		name="Path",
        description="Where to save the exported Mesh FBX",
		subtype="FILE_PATH",
		default="/"
	)
    
    anim_path: StringProperty(
		name="Folder",
        description="Folder to save the exported Anim FBX file in. The exported file will be named the current Action name",
		subtype="DIR_PATH",
		default="/"
	)
    
    all_anims_path: StringProperty(
		name="Path",
        description="Where save the exported Anim FBX file",
		subtype="FILE_PATH",
		default="/"
	)
    
    all_anims_path_folder: StringProperty(
		name="Path",
        description="Where save the exported Anim FBX files. 1 File Per Action: the exported files will be named after their Action names",
		subtype="DIR_PATH",
		default="/"
	)
    
    one_file_per_action: BoolProperty(
		name="1 File Per Action",
        description="When exporting all Actions, save each action to its own FBX file. Each file will be named after the Action's name",
		default=True
	)
    
    action_name_style: EnumProperty(
		name="Action File Names",
        description="When exporting Actions as FBX animations, the format to use for their file names",
		items=(('Action', "Action", "Actions FBX files are saved as just their name:\nActionName.fbx"),
               ('Name_Action', "Name_Action", "Actions FBX files are saved with a shared name with _ActionName:\nSharedName_ActionName.fbx"),
               ('Name-Action', "Name-Action", "Actions FBX files are saved with a shared name with -ActionName:\nSharedName-ActionName.fbx"),
               ('Name@Action', "Name@Action", "Actions FBX files are saved with a shared name with @ActionName:\nSharedName@ActionName.fbx"),
               ),
        default='Action',
	)
    
    action_name_sharedname: StringProperty(
		name="Shared Name",
        description="When exporting Actions as FBX animations, shared name that will be combined with the Action Name to make a file name",
		default=""
	)
    
    set_timeline_ignore_zero: BoolProperty(
		name="0->1",
        description="When true, Set Timeline will ignore a frame on frame zero, and set the start to frame one instead. Useful when previewing looping animations.\nNOTE: Quick Export functions will still export the full frame-range of the action",
		default=True
	)
    
    dummy_action_use_frame_range: BoolProperty(
        name="Manual Frame Range",
        default=False
    )
    
    dummy_action_frame_start: FloatProperty(
        name="Start",
        default=0.0
    )
    
    dummy_action_frame_end: FloatProperty(
        name="End",
        default=0.0
    )
    
    cur_action_shapekey_data: BoolProperty(
        name= "Export Shapekey Animation",
        description="Specify whether this Action should export Shapekey animation.\nWhen True, Quick Export will include deformed meshes with Shapekeys in the exported FBX animation",
        update=cur_action_shapekey_data_update
    )
    
    # Used by Quick Export operators when exporting one action per file
    action_export_list: CollectionProperty(
        type=ActionExport,
        name="Action Export List",
        options={'HIDDEN'},
    )
    
    min_two_frames: BoolProperty(
        name="Minimum 2 Frames",
        description="When exporting Actions, force frame range to be at least 2 frames long.\nFixes import issue with Unreal, where it cannot import 1 frame animations",
        default=True,
    )



class ProtoToolsPanel:
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'PROTO'


class PROTOTOOLS_PT_Export(bpy.types.Panel, ProtoToolsPanel):
    bl_label = 'PROTO FBX Export'
    
    def draw(self, context):
        layout = self.layout


class PROTOTOOLS_PT_Export_Quick(bpy.types.Panel, ProtoToolsPanel):
    bl_label = 'Quick Export Selected'
    bl_parent_id = "PROTOTOOLS_PT_Export"
    
    def draw(self, context):
        layout = self.layout
        prototools = context.scene.prototools
        
        col = layout.column()
        col.use_property_split = True
        col.use_property_decorate = False # keyframe/animate properties
        box = col.box()
        box.prop(prototools, "unit_scale_type")
        if prototools.unit_scale_type == "Custom":
            box.prop(prototools, "unit_scale_custom")
        
        layout.label(text="Export Mesh (static or skeletal):")
        
        split = layout.split(factor=0.3)
        labelCol = split.column()
        labelCol.alignment = 'RIGHT'
        valueCol = split.column()
        
        #labelCol.label(text="Mesh:")
        labelCol.label(text="File Path:")
        valueCol.prop(prototools, "mesh_path", text="")
        
        labelCol.label(text="")
        valueCol.operator("prototools.quick_export_mesh", text="Mesh", icon='EXPORT')
        
        layout.label(text="Export Animation:")
        
        split = layout.split(factor=0.3)
        labelCol = split.column()
        labelCol.alignment = 'RIGHT'
        valueCol = split.column()
        
        #labelCol.label(text="Active Action:")
        labelCol.label(text="Folder:")
        valueCol.prop(prototools, "anim_path", text="")
        
        labelCol.label(text="")
        valueCol.operator("prototools.quick_export_curaction", text="Active Action", icon='EXPORT')
        
        layout.label(text="Export All Animations:")
        
        split = layout.split(factor=0.3)
        labelCol = split.column()
        labelCol.alignment = 'RIGHT'
        valueCol = split.column()
        
        #labelCol.label(text="All Actions:")
        if prototools.one_file_per_action:
            labelCol.label(text="Folder:")
            valueCol.prop(prototools, "all_anims_path_folder", text="")
        else:
            labelCol.label(text="File Path:")
            valueCol.prop(prototools, "all_anims_path", text="")
            
        labelCol.label(text="")
        valueCol.prop(prototools, "one_file_per_action")
        
        labelCol.label(text="")
        valueCol.operator("prototools.quick_export_allactions", text="All Actions", icon='EXPORT')
        
        
class PROTOTOOLS_PT_Export_Quick_Options(bpy.types.Panel, ProtoToolsPanel):
    bl_label = 'Options'
    bl_parent_id = "PROTOTOOLS_PT_Export_Quick"
    
    def draw(self, context):
        layout = self.layout
        prototools = context.scene.prototools
        
        layout.use_property_split = True
        layout.use_property_decorate = False # keyframe/animate properties
        #layout.prop(prototools, "unit_scale_type")
        #if prototools.unit_scale_type == "Custom":
        #    layout.prop(prototools, "unit_scale_custom")
        layout.prop(prototools, "root_bone_name")
        layout.prop(prototools, "apply_modifiers")
        layout.prop(prototools, "action_name_style")
        if prototools.action_name_style != "Action":
            layout.prop(prototools, "action_name_sharedname")
        layout.prop(prototools, "min_two_frames")
        

class PROTOTOOLS_PT_ManualExport(bpy.types.Panel, ProtoToolsPanel):
    bl_label = 'Manual Export'
    bl_parent_id = "PROTOTOOLS_PT_Export"
    bl_options = {'DEFAULT_CLOSED'}
    
    def draw(self, context):
        layout = self.layout
        
        # Create 3 columns, with wide center column, to add padding
        # around the operator button
        split = layout.split(factor=0.1)
        gap = split.column()
        col = split.column()
        split = col.split(factor=0.88888)
        col = split.column()
        row = col.row(align=True)
        
        row.operator("proto_export_scene.fbx", text="PROTO FBX Export...")




class PROTOTOOLS_PT_ActionSelector(bpy.types.Panel, ProtoToolsPanel):
    bl_label = 'Quick Action Select'
    
    def draw(self, context):
        layout = self.layout
        prototools = context.scene.prototools
        
        layout.label(text="Actions", icon="ACTION")
        
        # Create 3 columns, with wide center column, to add padding
        # around the operator button
        #split = layout.split(factor=0.1)
        #gap = split.column()
        #col = split.column()
        #split = col.split(factor=0.88888)
        #col = split.column()
        #row = col.row(align=True)
        
        col = layout.column()
        row = col.row(align=True)
        
        if context.active_object != None and context.active_object.animation_data != None:
            row.template_ID(context.active_object.animation_data, "action", new="prototools.actionnew", unlink="prototools.actionunlink")
            if context.active_object.animation_data.action != None:
                # Delete action button
                row.operator("prototools.actiondelete", text="", icon="TRASH")
                
        else:
            row.operator("prototools.makeanimdata", icon="PLUS")
        
        # Frame range, set timeline widgets
        if context.active_object != None and context.active_object.animation_data != None and context.active_object.animation_data.action != None:
            action = context.active_object.animation_data.action
            col.prop(action, "use_frame_range")
            col2 = col.column()
            col2.active = action.use_frame_range
            row = col2.row(align=True)
            row.prop(action, "frame_start", text="Start")
            row.prop(action, "frame_end", text="End")
            
            col3 = col.column()
            split = col3.split(factor=0.7)
            col3_1 = split.column()
            col3_2 = split.column()
            col3_1.operator("prototools.timelinetoaction")
            col3_2.prop(prototools, "set_timeline_ignore_zero")
            
            # If this custom property does not exist on the action, we display a "dummy" property to set it for the first time
            if "Export Shapekey Animation" in action:
                col.prop(action, '["Export Shapekey Animation"]')
            else:
                col.prop(prototools, "cur_action_shapekey_data")
        else:
            # Dummy version that is always disabled
            colDisabled = col.column()
            colDisabled.active = False
            colDisabled.prop(prototools, "dummy_action_use_frame_range")
            col2 = colDisabled.column()
            row = col2.row(align=True)
            row.prop(prototools, "dummy_action_frame_start", text="Start")
            row.prop(prototools, "dummy_action_frame_end", text="End")
            
            split = col2.split(factor=0.7)
            col2_1 = split.column()
            col2_2 = split.column()
            row = col2.row(align=False)
            col2_1.operator("prototools.timelinetoaction")
            col2_2.prop(prototools, "set_timeline_ignore_zero")
            
            # always show dummy
            colDisabled.prop(prototools, "cur_action_shapekey_data")
        

class PROTOTOOLS_PT_VertexColors(bpy.types.Panel, ProtoToolsPanel):
    bl_label = 'Vertex Color Tools'
    bl_options = {'DEFAULT_CLOSED'}
    
    def draw(self, context):
        layout = self.layout
        
        layout.label(text="Fill Vertex Color", icon="VPAINT_HLT")
        
        # Create 3 columns, with wide center column, to add padding
        # around the operator button
        split = layout.split(factor=0.1)
        gap = split.column()
        col = split.column()
        split = col.split(factor=0.88888)
        col = split.column(align=True)
        row = col.row(align=True)
        row.scale_y = 1.2
        
        row = col.row(align=True)
        row.scale_y = 1.2
        
        op_r = row.operator("prototools.fill_vertex_color", text="R")
        op_r.channel = 0
        op_r.weight = 1.0
        op_g = row.operator("prototools.fill_vertex_color", text="G")
        op_g.channel = 1
        op_g.weight = 1.0
        op_b = row.operator("prototools.fill_vertex_color", text="B")
        op_b.channel = 2
        op_b.weight = 1.0
        
        row = col.row(align=True)
        row.scale_y = 1.2
        
        op_r = row.operator("prototools.fill_vertex_color", text="Clear")
        op_r.channel = 0
        op_r.weight = 0.0
        op_g = row.operator("prototools.fill_vertex_color", text="Clear")
        op_g.channel = 1
        op_g.weight = 0.0
        op_b = row.operator("prototools.fill_vertex_color", text="Clear")
        op_b.channel = 2
        op_b.weight = 0.0
        
        
        layout.label(text="Vertex Group to Vertex Color", icon="GROUP_VERTEX")
        
        # Create 3 columns, with wide center column, to add padding
        # around the operator button
        split = layout.split(factor=0.1)
        gap = split.column()
        col = split.column()
        split = col.split(factor=0.88888)
        col = split.column(align=True)
        row = col.row(align=True)
        row.scale_y = 1.2
        
        label_str = "Group: none"
        if context.active_object != None and context.active_object.vertex_groups.active != None:
            label_str = "Group: " + context.active_object.vertex_groups.active.name
        row.label(text=label_str)
        
        row = col.row(align=True)
        row.scale_y = 1.2
        
        op_r = row.operator("prototools.vertex_group_to_vertex_color", text="R")
        op_r.channel = 0
        op_g = row.operator("prototools.vertex_group_to_vertex_color", text="G")
        op_g.channel = 1
        op_b = row.operator("prototools.vertex_group_to_vertex_color", text="B")
        op_b.channel = 2


def register():
    bpy.utils.register_class(ActionExport)
    bpy.utils.register_class(ProtoToolsProperties)
    bpy.utils.register_class(PROTOTOOLS_PT_Export)
    bpy.utils.register_class(PROTOTOOLS_PT_Export_Quick)
    bpy.utils.register_class(PROTOTOOLS_PT_Export_Quick_Options)
    bpy.utils.register_class(PROTOTOOLS_PT_ManualExport)
    bpy.utils.register_class(PROTOTOOLS_PT_ActionSelector)
    bpy.utils.register_class(PROTOTOOLS_PT_VertexColors)
    
    bpy.types.Scene.prototools = bpy.props.PointerProperty(type=ProtoToolsProperties)
    

def unregister():
    bpy.utils.unregister_class(ActionExport)
    bpy.utils.unregister_class(ProtoToolsProperties)
    bpy.utils.unregister_class(PROTOTOOLS_PT_Export)
    bpy.utils.unregister_class(PROTOTOOLS_PT_Export_Quick)
    bpy.utils.unregister_class(PROTOTOOLS_PT_Export_Quick_Options)
    bpy.utils.unregister_class(PROTOTOOLS_PT_ManualExport)
    bpy.utils.unregister_class(PROTOTOOLS_PT_ActionSelector)
    bpy.utils.unregister_class(PROTOTOOLS_PT_VertexColors)
    
    del bpy.types.Scene.prototools
    