# PROTO Tools script
# 2024 PROTOWLF, Licensed under GPL-3.0

import bpy
from bpy.props import (
    IntProperty,
    FloatProperty,
    BoolProperty,
)


def get_paintable_loops(ob, skip_mask):
    loops = []
    # Check mask
    if ob.data.use_paint_mask and not skip_mask:
        # Iterate through all faces, and store a list of selected faces' loops
        for face in ob.data.polygons:
            if face.select:
                for index in face.loop_indices:
                    loops.append(ob.data.loops[index])
    else:
        # All loops
        loops = ob.data.loops
    
    return loops

class ProtoTools_VertexGroupToVertexColor(bpy.types.Operator):
    """Copy the weight of the current Vertex Group into the given channel of the current Vertex Color Attribute"""
    bl_idname = "prototools.vertex_group_to_vertex_color"
    bl_label = "Vertex Group To Vertex Color)"
    bl_options = {'REGISTER', 'UNDO'}
    
    channel: bpy.props.IntProperty()
    invert: bpy.props.BoolProperty(default=False)
    skip_mask: bpy.props.BoolProperty(default=False)
    
    @classmethod
    def poll(self, context):
        ob = context.active_object
        if ob != None and ob.type == "MESH" and ob.vertex_groups.active != None:
            return True
        return False
    
    def execute(self, context):
        prototools = context.scene.prototools
        
        # Clamp channel property
        if self.channel < 0:
            self.channel = 0
        elif self.channel > 2:
            self.channel = 2
        
        # Active Object and active Vertex Group assumed to exist
        ob = context.active_object
        vertex_group = ob.vertex_groups.active
        
        # Make Vertex Color Attribute if one doesn't exist
        if ob.data.attributes.active_color is None:
            bpy.ops.geometry.color_attribute_add(name='Col', domain='CORNER', data_type='BYTE_COLOR')
        
        # Verify active Vertex Color Attribute
        if ob.data.attributes.active_color.domain != 'CORNER':
            self.report({'ERROR'}, "Invalid Color Attribute - Active color attribute must be of type Face Corner")
            return {'CANCELLED'}
        vertex_colors = ob.data.attributes.active_color.data
        
        loops = get_paintable_loops(ob, self.skip_mask)
        
        # Copy weight
        for loop in loops:
            try:
                weight = 1.0 - vertex_group.weight(loop.vertex_index) if self.invert else vertex_group.weight(loop.vertex_index)
            except:
                weight = 1.0 if self.invert else 0.0
            
            vertex_colors[loop.index].color[self.channel] = weight
        
        return {'FINISHED'}
    

class ProtoTools_FillVertexColor(bpy.types.Operator):
    """Set the given channel of the current Vertex Color Attribute to the given weight"""
    bl_idname = "prototools.fill_vertex_color"
    bl_label = "Fill Vertex Color"
    bl_options = {'REGISTER', 'UNDO'}
    
    channel: bpy.props.IntProperty()
    weight: bpy.props.FloatProperty()
    skip_mask: bpy.props.BoolProperty(default=False)
    
    @classmethod
    def poll(self, context):
        ob = context.active_object
        if ob != None and ob.type == "MESH" and ob.data.vertex_colors.active != None:
            return True
        return False
    
    def execute(self, context):
        prototools = context.scene.prototools
        
        # Active Object assumed to exist
        ob = context.active_object
        
        # Clamp channel property
        if self.channel < 0:
            self.channel = 0
        elif self.channel > 2:
            self.channel = 2
        
        # Verify active Vertex Color Attribute
        if ob.data.attributes.active_color.domain != 'CORNER':
            self.report({'ERROR'}, "Invalid Color Attribute - Active color attribute must be of type Face Corner")
            return {'CANCELLED'}
        vertex_colors = ob.data.attributes.active_color.data
        
        loops = get_paintable_loops(ob, self.skip_mask)
        
        # Set weight
        for loop in loops:
            vertex_colors[loop.index].color[self.channel] = self.weight
        
        return {'FINISHED'}


def register():
    bpy.utils.register_class(ProtoTools_VertexGroupToVertexColor)
    bpy.utils.register_class(ProtoTools_FillVertexColor)
    

def unregister():
    bpy.utils.unregister_class(ProtoTools_VertexGroupToVertexColor)
    bpy.utils.unregister_class(ProtoTools_FillVertexColor)

